unit image_button;
{< TImage descendant to represent a button.}

{$IFDEF fpc}
  {$mode objfpc}{$H+}
{$ENDIF}


interface

uses
  SysUtils
  ,Windows
  ,Classes
  ,Controls
  ,ExtCtrls
  ,Graphics
  
  ;

type

  {Base Image Button Object.}
  TDTImageButton = class(TImage)
  private
    FCaption: string;
    FButtonFont: TFont;
    FUpImageName: string;
    FDownImageName: string;
    FData: TObject;
    FDataString: string;
    FDataObject: TObject;
    procedure SetCaption(const Value: string);
    procedure SetButtonFont(const Value: TFont);
    procedure SetDownImageName(const Value: string);
    procedure SetUpImageName(const Value: string);
    procedure SetData(const Value: TObject);
    procedure SetDataObject(const Value: TObject);
    procedure SetDataString(const Value: string);
  protected
    { Draws the buttons caption on the image. }
    procedure   DrawCaption;
    procedure   DrawCaptionDirectly;
    { Handles mousdown event to change image.}
    procedure   HandleMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    { Handles mouseup event to change image back. }
    procedure   HandleMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Paint;override;
  public
    constructor Create(AOwern: TComponent); reintroduce; overload;
    destructor  Destroy; override;
    // ---> methods
    procedure   Init;
    // ---> Properties
    property    Caption: string read FCaption write SetCaption;
    property    ButtonFont: TFont read FButtonFont write SetButtonFont;
    property    UpImageName: string read FUpImageName write SetUpImageName;
    property    DownImageName: string read FDownImageName write SetDownImageName;
    property    DataObject: TObject read FDataObject write SetDataObject;
    property    DataString: string read FDataString write SetDataString;
end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('DataTrak', [TDTImageButton]);
end;

{ DTImageButton }

constructor TDTImageButton.Create(AOwern: TComponent);
begin
  inherited;
  FButtonFont := TFont.Create;
  FButtonFont.Size := 10;
  FButtonFont.Color := clwhite;

  //AutoSize := true;

  {$IFDEF fpc}
    OnMouseDown := @HandleMouseDown;
    OnMouseUp := @HandleMouseUp;
  {$ELSE}
    OnMouseDown := HandleMouseDown;
    OnMouseUp := HandleMouseUp;
  {$ENDIF}
end;

destructor TDTImageButton.Destroy;
begin
  FButtonFont.Free;
  inherited;
end;

procedure TDTImageButton.DrawCaption;
var
  lRect: TRect;
begin
  // setup rect
  lRect.Left := 10;
  lRect.Top := 10;
  lRect.Right := Width -10;
  lRect.Bottom := Height -10;
  // setup font for canvas
  //Canvas.Pen.Style := psClear; // <-- tried this as well
  Canvas.Font.Assign(FButtonFont);
  Canvas.Brush.Style := bsClear;
  
  //SetBkMode(Canvas.Handle, Windows.TRANSPARENT);
  //Canvas.Color := clNone;
  
  //Canvas.TextOut(0,0, FCaption);
  //self.Canvas.TextRect(lrect, 10,10, FCaption, lTextStyle);
  DrawText(Canvas.Handle, PChar(FCaption), -1, lRect,
    DT_WORDBREAK or DT_VCENTER or DT_CENTER );
    
end;

procedure TDTImageButton.DrawCaptionDirectly;
begin

end;

procedure TDTImageButton.HandleMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  lBmp: TBitmap;
begin
  if Button = mbLeft then
    begin
      lBmp := TBitMap.Create;
      try
      // draw bitmap
      lBmp.LoadFromFile('down.bmp');
      Picture.Bitmap.Assign(lBmp);
      DrawCaption;
      finally;
        lBmp.free;
        end;
    end;
end;

procedure TDTImageButton.HandleMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  lBmp: TBitmap;
begin
  if Button = mbLeft then
    begin
      lBmp := TBitMap.Create;
      try
      // draw bitmap
      lBmp.LoadFromFile('up.bmp');
      Picture.Bitmap.Assign(lBmp);
      DrawCaption;
      finally;
        lBmp.free;
        end;
    end;
end;

procedure TDTImageButton.Paint;
begin
  inherited Paint;
end;

procedure TDTImageButton.Init;
var
  lBmp: TBitmap;
begin
  lBmp := TBitMap.Create;
  try
  // draw bitmap
  lBmp.LoadFromFile('up.bmp');
  Picture.Bitmap.Assign(lBmp);
  DrawCaption;
  finally;
    lBmp.free;
    end;      ;
  DrawCaption;
end;

procedure TDTImageButton.SetButtonFont(const Value: TFont);
begin
  FButtonFont := Value;
end;

procedure TDTImageButton.SetCaption(const Value: string);
begin
  FCaption := Value;
end;

procedure TDTImageButton.SetData(const Value: TObject);
begin
  FData := Value;
end;

procedure TDTImageButton.SetDataObject(const Value: TObject);
begin
  FDataObject := Value;
end;

procedure TDTImageButton.SetDataString(const Value: string);
begin
  FDataString := Value;
end;

procedure TDTImageButton.SetDownImageName(const Value: string);
begin
  FDownImageName := Value;
end;

procedure TDTImageButton.SetUpImageName(const Value: string);
begin
  FUpImageName := Value;
end;

end.
